# -*- coding: utf-8 -*-

import os
import subprocess

import yaml

import yml_check
import jinja_check


def find_files(filenames, check_files, include=True):

    if include:
        for fname in check_files:
            if fname not in filenames:
                yield "{0} not in directory - please add".format(fname)
    elif not include:
        for fname in check_files:
            if fname in filenames:
                yield "{0} in directory - please remove".format(fname)


def run(*args):
    p = subprocess.Popen(args, stdout=subprocess.PIPE)
    out, _ = p.communicate()
    return out


def do_check_pep8(files, status):
    """
    Run the python pep8 tool against the filst of supplied files.
    Append any linting errors to the returned status list

    Args:
        files (str): list of files to run pep8 against
        status (list): list of pre-receive check failures to eventually print
                       to the user

    Returns:
       status list of current pre-redeive check failures. Might be an empty
       list.
    """
    for file_name in files:

        args = ['flake8', '--max-line-length=120', '{0}'.format(file_name)]
        output = run(*args)

        if output:
            status.append("Python PEP8/Flake8: {0}: {1}".format(file_name,
                                                                output))

    return status


def do_check(func, files, status):
    """
    Generic do_check helper method

    Args:
        func (function): Specific function to call
        files (list): list of files to run against
        status (list): list of pre-receive check failures to eventually print
                       to the user

    Returns:
       status list of current pre-redeive check failures. Might be an empty
       list.
    """

    for file_name in files:
        with open(file_name, 'r') as f:
            output = func.parse(f.read(), file_name)

        if output:
            status.append("{0}: {1}".format(file_name, output))

    return status


def get_directory_tree():
    for root, dirs, files in os.walk(".", topdown=True):
        for name in files:
            yield(os.path.join(root, name))


def main():
    try:
        mypath = os.path.join(os.path.expanduser('~'), '.plaidrc')
        with open(mypath, 'r') as f:
            yaml_dict = yaml.load(f.read())
    except IOError:
        blank_rc_file = """---

include_files: []
exclude_files: []
        """
        with open(mypath, 'w') as fw:
            fw.write(blank_rc_file)
        yaml_dict = {}

    files = list(get_directory_tree())

    yml_files = [x for x in files if x.endswith('.yml')]
    jinja_files = [x for x in files if x.endswith('.j2')]
    py_files = [x for x in files if x.endswith('.py')]

    status = []
    status = do_check(yml_check, yml_files, status)
    status = do_check(jinja_check, jinja_files, status)
    status = do_check_pep8(py_files, status)

    if status:
        print("****************************\n\n"
              "Ansible checks FAILED:\n"
              "Please study the output and please resolve\n\n"
              "***************************\n")
        for msg in status:
            print(msg)

    filenames = [os.path.basename(x) for x in files]

    include_files = yaml_dict.get('include_files')
    if include_files:
        for result in find_files(filenames, include_files):
            print(result)

    exclude_files = yaml_dict.get('exclude_files')
    if exclude_files:
        for result in find_files(filenames, exclude_files, include=False):
            print(result)



"""
Generic jinja2 parser
"""


import sys

from jinja2 import meta, Environment


def find_undeclared_variables(template, file_name):
    '''
    Finds undeclared variables in a jinja template. Ths checks for the use case
    when a develop inadvertently adds a file to the templates directory

    Args:
        template (jinja2.nodes.Template'): Parsed Jinja template

    Returns:
        str: Error string
    '''

    undecl_vars = meta.find_undeclared_variables(template)

    if not undecl_vars:
        return ("ERROR: Jinja2 Template File:"
                "File {0} has no variable to substitute in the template!\n"
                .format(file_name))


def parse(file_contents, file_name):
    '''
    Takes a list of files which are assumed to be jinja2 templates and tries to
    parse the contents of the files

    Args:
        file_contents (str): File contents of a jinja file

    Raises:
        Exception: An exception is raised if the contents of the file cannot be
                   parsed.
    '''

    env = Environment()
    result = ""
    try:
        ast = env.parse(file_contents)
        result = find_undeclared_variables(ast, file_name)
    except Exception:
        _, exc_value, _ = sys.exc_info()
        result += "ERROR: Jinja2 Template File: {0}".format(file_name)
        result += repr(exc_value) + '\n'

    return result



#!/usr/bin/env python
"""
Generic yaml parser
"""


import sys
import yaml


def parse(file_contents, file_name):
    """
    This takes a list of filenames and their paths of expected yaml files and
    tried to parse them, erroring if there are any parsing issues.

    Args:
        file_contents (str): Contents of a yml file

    Raises:
        yaml.parser.ParserError: Raises an error if the file contents cannot be
                                 parsed and interpreted as yaml
    """

    try:
        yaml.load(file_contents)
    except Exception:

        _, exc_value, _ = sys.exc_info()
        return("Cannot Parse: {file_name}: \n {exc_value}"
               .format(file_name=file_name, exc_value=exc_value))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'click>=6.0',
    'flake8>=2.6.0',
    'Jinja2>=2.9.4',
    'pyflakes>=1.2.3',
    'PyYAML>=3.11'
]

test_requirements = [
    # TODO: put package test requirements here
]

setup(
    name='plaid',
    version='0.1.5',
    description="Python Ansible role checker",
    long_description=readme + '\n\n' + history,
    author="Scott Ison",
    author_email='sqrvrtx@gmail.com',
    url='https://github.com/sqrvrtx/plaid',
    packages=[
        'plaid',
    ],
    package_dir={'plaid':
                 'plaid'},
    entry_points={
        'console_scripts': [
            'plaid=plaid.cli:main'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    license="MIT license",
    zip_safe=False,
    keywords='plaid',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
